@@ -409,6 +409,7 @@ class Sparse3DNA(nn.Module):
         heads = 8,
         dim_head = 64,
         dropout = 0.,
+        causal = False,
         query_num_frames_chunk = None
     ):
         super().__init__()
@@ -435,6 +436,11 @@ class Sparse3DNA(nn.Module):
 
         # precalculate causal mask
 
+        self.causal = causal
+
+        if not causal:
+            return
+
         indices = torch.arange(max_num_tokens)
         shaped_indices = rearrange(indices, '(f h w) -> 1 1 f h w', f = max_frames, h = fmap_size, w = fmap_size)
         padded_indices = F.pad(shaped_indices, (kernel_size // 2,) * 6, value = max_num_tokens) # padding has value of max tokens so to be masked out
@@ -519,10 +525,10 @@ class Sparse3DNA(nn.Module):
 
             # causal mask
 
-            mask_value = -torch.finfo(sim.dtype).max
-            causal_mask = rearrange(causal_mask, 'i j -> 1 i j')
-
-            sim = sim.masked_fill(causal_mask, mask_value)
+            if exists(causal_mask):
+                mask_value = -torch.finfo(sim.dtype).max
+                causal_mask = rearrange(causal_mask, 'i j -> 1 i j')
+                sim = sim.masked_fill(causal_mask, mask_value)
 
             # attention
 
@@ -538,7 +544,15 @@ class Sparse3DNA(nn.Module):
         frames_per_chunk = min(self.query_num_frames_chunk, num_frames)
         chunk_size = frames_per_chunk * tokens_per_frame
 
-        for ind, q_chunk in enumerate(q.split(chunk_size, dim = 1)):
+        q_chunks = q.split(chunk_size, dim = 1)
+
+        if self.causal:
+            causal_mask = self.causal_mask[:(n - 1)]
+            causal_mask_chunks = causal_mask.split(chunk_size, dim = 0)
+
+        for ind in range(len(q_chunks)):
+            q_chunk = q_chunks[ind]
+            causal_mask_chunk = causal_mask_chunks[ind] if self.causal else None
 
             # slice the keys and values to the appropriate frames, accounting for padding along frames dimension
 
@@ -548,24 +562,16 @@ class Sparse3DNA(nn.Module):
 
             k_slice, v_slice = map(lambda t: t[:, :, kv_frame_range], (k, v))
 
-            # slice causal mask to the appropriate query chunk windows - no padding need to be accounted for
-
-            mask_start_pos = ind * chunk_size
-            mask_end_pos = mask_start_pos + q_chunk.shape[1]
-            mask_range = slice(mask_start_pos, mask_end_pos)
-
-            causal_mask_slice = self.causal_mask[mask_range]
-
             # calculate output chunk
 
             out_chunk = attend(
                 q = q_chunk,
                 k = k_slice,
                 v = v_slice,
-                causal_mask = causal_mask_slice,
                 k_bos = k_bos,
                 v_bos = v_bos,
-                kernel_size = kernel_size
+                kernel_size = kernel_size,
+                causal_mask = causal_mask_chunk
             )
 
             out.append(out_chunk)
@@ -614,7 +620,7 @@ class Transformer(nn.Module):
 
         for _ in range(depth):
             if sparse_3dna_attn:
-                self_attn = Sparse3DNA(dim = dim, heads = heads, dim_head = dim_head, kernel_size = sparse_3dna_kernel_size, video_shape = sparse_3dna_video_shape, query_num_frames_chunk = sparse_3dna_query_num_frames_chunk)
+                self_attn = Sparse3DNA(dim = dim, heads = heads, dim_head = dim_head, causal = causal, kernel_size = sparse_3dna_kernel_size, video_shape = sparse_3dna_video_shape, query_num_frames_chunk = sparse_3dna_query_num_frames_chunk)
             else:
                 self_attn = Attention(dim = dim, heads = heads, dim_head = dim_head, causal = causal, dropout = attn_dropout)
 
